#pragma once
#include <windows.h>
#include "Form2.h"
#include "Form1.h"
#include "Town.h"

namespace BARGAME {

	using namespace System;
	using namespace System::ComponentModel;
	using namespace System::Collections;
	using namespace System::Windows::Forms;
	using namespace System::Data;
	using namespace System::Drawing;
	using namespace System::Threading;
	
public ref class Form2 : public System::Windows::Forms::Form{

public:
	//! \brief Form2 Constructor
	/*!
	* \details Description: Initializes the components of Form2 and private variables with default values, and calls Town's constructor with input data from Simulation Button.
	* \param numagents Number of agents
	* \param numbars1 Number of bars
	* \param barcap[] Array holding bar capacities
	* \param ispercentcap Bool representing if capacity is of type percent 
	* \param *loading Integer used for progress bar that gives a percent of completion
	********************************************************************/
	Form2(double numagents,int numbars1,int barcap[],bool ispercentcap, int* loading);
	//! \brief Form2 Destructor
	/*!
	* \details Description: Deletes components of Form2.
	********************************************************************/
	~Form2();
	
protected:
		//! \brief String that holds output file name.
		String^ filename;
		//! \brief Integer which holds number of bars.
		int numbars;
		//! \brief Integer which holds speed of round.
		int sleepytime;
		//! \brief Integer which represents left side of displayed chart.
		int selectedindex1;
		//! \brief Integer which represents right side of displayed chart.
		int selectedindex2;
		//! \brief Integer that tells the thread to stop.
		int stop;
		//! \brief Integer which holds current round number.
		int turn;
		//! \brief Town pointer used for backend.
		Town *town;
		//! \brief Array which holds all data needed for all charts. Offsets are used to give specific data to chart methods.
		double *ultimategraph;
		//! \brief graphPtr object used for backend.
		graphPtr t;
		//! \brief Thread which backend is run on.
		Thread^ backendthread;
		//! \brief Delegate for thread safety.
		delegate void SetTextDelegate(String^ text);
		
		//! \brief Chart Method
		/*!
		* \details Description: This method calls corresponding methods to create charts based on what is selected from dropdowns.
		* \param *data Array which holds data for charts.
		* \pre Chart data exists.
		* \post Chart is displayed in GUI.
		********************************************************************/
		void runner(double *data);
		//! \brief Winning Bars Method
		/*!
		* \details Description: This method uses the ChartDirector API to create the graph for "Number of Bars that Won" option.
		* \param data Array which holds data for charts.
		* \pre Chart data exists.
		* \post Chart is created.
		********************************************************************/
		void winningbars(double data[],int size,int leftorright );
		//! \brief Winners Method
		/*!
		* \details Description: This method uses the ChartDirector API to create the graph for "Number of Winners" option.
		* \param data Array which holds data for charts.
		* \param size Number of elements to be passed to ChartDirector API.
		* \param leftorright Integer which tells the method to display graph on either right or left side of GUI.
		* \pre Chart data exists.
		* \post Chart is created.
		********************************************************************/
		void winners(double data[],int size,int leftorright );
		//! \brief Average Strategy Scores Method
		/*!
		* \details Description: This method uses the ChartDirector API to create the graph for "Average Score of All Strategies" option.
		* \param data Array which holds data for charts.
		* \param size Number of elements to be passed to ChartDirector API.
		* \param leftorright Integer which tells the method to display graph on either right or left side of GUI.
		* \pre Chart data exists.
		* \post Chart is created.
		********************************************************************/
		void avgstratscores(double data[],int size,int leftorright );
		//! \brief Best Strategy Method
		/*!
		* \details Description: This method uses the ChartDirector API to create the graph for "Best Strategy's Scores" option.
		* \param data Array which holds data for charts.
		* \param size Number of elements to be passed to ChartDirector API.
		* \param leftorright Integer which tells the method to display graph on either right or left side of GUI.
		* \pre Chart data exists.
		* \post Chart is created.
		********************************************************************/
		void beststrat(double data[],int size,int leftorright);
		//! \brief Bar Distribution Method
		/*!
		* \details Description: This method uses the ChartDirector API to create the graph for "Wins per Bar" option.
		* \param data Array which holds data for charts.
		* \param size Number of elements to be passed to ChartDirector API.
		* \param leftorright Integer which tells the method to display graph on either right or left side of GUI.
		* \pre Chart data exists.
		* \post Chart is created.
		********************************************************************/
		void bardist(double data[],int size,int leftorright);
		//! \brief Number of Deaths Method
		/*!
		* \details Description: This method uses the ChartDirector API to create the graph for "Number of Deaths" option.
		* \param data Array which holds data for charts.
		* \param size Number of elements to be passed to ChartDirector API.
		* \param leftorright Integer which tells the method to display graph on either right or left side of GUI.
		* \pre Chart data exists.
		* \post Chart is created.
		********************************************************************/
		void numdeaths(double data[],int size,int leftorright);
		//! \brief Age Distribution Method
		/*!
		* \details Description: This method uses the ChartDirector API to create the graph for "Age Distribution" option.
		* \param data Array which holds data for charts.
		* \param size Number of elements to be passed to ChartDirector API.
		* \param leftorright Integer which tells the method to display graph on either right or left side of GUI.
		* \pre Chart data exists.
		* \post Chart is created.
		********************************************************************/
		void agedist(double data[],int size,int leftorright);
		//! \brief Start/Resume Simulation Button
		/*!
		* \details Description: Button that retrieves speed and selected dropdown options and starts thread to run backend. Also acts as a pause button which stops and starts backend thread.
		* \pre Speed is set and charts are chosen from dropdown.
		* \post Chart is displayed and updated in realtime in GUI.
		********************************************************************/
		System::Void button2_Click(System::Object^  sender, System::EventArgs^  e);
		//! \brief Backend Method
		/*!
		* \details Description: Method passed to thread that runs backend, updates speed if changed, creates data for graphs, calls runner method, and dumps graph data to output file.
		* \pre Backend succesfully intialized from Form2 constructor.
		* \post Chart data is created and sent to runner(), output file is created and populated.
		********************************************************************/
		void ThreadProcSafe();
		//! \brief Round Number Method
		/*!
		* \details Description: Method that updates current round number in real time.
		* \param text String containing current round number.
		* \pre Backend thread started.
		* \post Current round number displayed.
		********************************************************************/
		void SetText(String^ text);
		//! \brief Marshal String Method
		/*!
		* \details Description: Method that converts System::String^ variable to string varib=able.
		* \param text String containing current round number.
		* \pre None.
		* \post System::String^ has been converted to string.
		********************************************************************/
		void MarshalString ( String ^ s, string& os );
		//! \brief Left Drop-down Event
		/*!
		* \details Description: Event that changes displayed chart type when new chart is selected.
		* \pre Form2 is created.
		* \post Selected chart is displayed.
		********************************************************************/
		void ComboBox1_SelectedIndexChanged( Object^ sender, System::EventArgs^ e );
		//! \brief Right Drop-down Event
		/*!
		* \details Description: Event that changes displayed chart type when new chart is selected.
		* \pre Form2 is created.
		* \post Selected chart is displayed.
		********************************************************************/
		void ComboBox2_SelectedIndexChanged( Object^ sender, System::EventArgs^ e );
		//! \brief Form2 Close Event
		/*!
		* \details Description: Event that stops backend thread and deletes dynamically allocated memory.
		* \pre Form2 is created.
		* \post Form2 exits.
		********************************************************************/		
		System::Void Form2_FormClosed(System::Object^  sender, System::Windows::Forms::FormClosedEventArgs^  e);
}
}