#ifndef STRATEGY_H
#define STRATEGY_H

//! \brief Strategy Class.
/*!
* This class is the computaional unit of the Strategies utilized by the Agent Objects. It has 
* simple get functions but also functions which updates and chooses stategies based on their respective score
* where their score respresents a strategies performance throughout the simulation
*********************************************************************************************/
class Strategy{
public:

	/*! \brief Default Strategy Constructor
	*
	* Default constructor NOT utilized within code.
	******************************************************************************************/
	Strategy();
	
	/*! \brief Strategy Destructor
	*
	* Destructor
	******************************************************************************************/
	~Strategy();

	/*! \brief Strategy Constructor
	* \details Description: Strategy Constructor that is utilized by the Agent class.
	* \param numbars indicates how many bars are present for the given simulation profile.
	* \pre 0 \< numbars \< 256
	* \post Integer array st that holds Strategies has been initialized with a random values
	**********************************************************************/
	Strategy(int numbars);
	
	/*! \brief Strategy Copy Constructor
	* \details Description: Strategy Constructor that is utilized by the Agent class to copy 
	* a set of Strategy's
	* \param s is a reference pointer to another Strategy object (usually contained within an Agent)
	* \pre 0 \< numbars \< 256
	* \post Integer array st that holds Strategies has been initialized with a random values
	**********************************************************************/
	Strategy(Strategy *s);
	
	/*! \brief Returns a Strategy's current score.
	* \details Description: Returns a Double which represents a Strategy's current score.
	* \pre None.
	* \post None.
	**********************************************************************/
	double getScore();
		
	/*! \brief
	* \details Description: Returns which bar number the Agent will go to for a given round. This 
	* is our hashing function.
	* \param sti short term index
	* \pre None.
	* \post None.
	*********************************************************************/
	int getBar(long sti);
	/*! \brief Updates a given STM strategy i
	* \details Description: A strategies previous score is updated for a given turn.
	* \param i One of the STM stragtegies stored in strats[] \note In this case our application only keeps 
	* track of the last 3 strategies.
	* \pre None.
	* \post Private variable "score" is updated.
	**********************************************************************/
	double updateScore(int i);
		
	/*! \brief returns the strategy to be used by Agent for a given turn.
	* \details Description: Gets either strategy 0, 1, or, 2 because each Agent has 3 strategies to choose from.
	* \pre 0 \< shortterm \< 254^4 - 1
	* \post Returns which strategy number(0,1,2) the Agent is going to utilize for the upcomming turn.
	*********************************************************************/
	int getStrat(int shortterm);
	
	/*! \brief Randomizes keys 
	* \details Description: Randomizes keys that are greater than 2048 in order to attempt to minimize memory usage. 
	* Is an explicit Hash function.
	* \param shortterm the value that is greater than 2048 (2^11)
	* \pre shortterm > 2048
	* \post new number maps to smaller number than 2048
	***********************************************************************/
	int hash(int shortterm);
	
	
	/*! \brief Short Term Structure
	*
    * The structure used to store all possible STM combinations
    */
	int st[2048];
	
	/*! \brief A strategies Score
	*
    * Contains the score of a strategy 
	* \note scaled on values from 0 (worst) to 10 (best)
    */
	double score;

};

#endif